EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFileVersions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates new configuration file versions for each instance specified in the ConfigurationFiles_Xml for the given configuration file.'
GO

IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFileVersions]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFileVersions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFileVersions]
(
	@ConfigurationFile_Id INT,
	@ConfigurationFiles_Xml XML,
	@ReleaseNumbers_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	-- Do nothing if there are no instances specified
	IF COALESCE(@ConfigurationFiles_Xml.exist('/ConfigFiles/Version/@Instance_Name'), 0) = 0
		RETURN

	EXEC [ConfigurationFiles_FillVersionGaps] @ConfigurationFile_Id


	/*******************************************************************
	 * Now versions should be consistent, so figure out next version
	 ******************************************************************/
	DECLARE @Version_Number INT
	    SET @Version_Number = COALESCE((SELECT MAX([Version_Number])
		                                  FROM [ConfigurationFileVersions]
								         WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id), 0) + 1

	/* XML format example
		<?xml version="1.0" encoding="utf-16"?>
		<ConfigFiles>
			<Version
				Instance_Name="Default"
				VersionNotes_Text="Notes..."
				File_Bytes="::base64::"
			/>
		</ConfigFiles>
	*/

	INSERT INTO [ConfigurationFileVersions]
	(
		[ConfigurationFile_Id],
		[Instance_Name],
		[Version_Number],
		[VersionNotes_Text],
		[File_Date],
		[File_Bytes],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	SELECT CFI.[ConfigurationFile_Id],
	       CFI.[Instance_Name],
		   @Version_Number,
		   CFV_Input.[VersionNotes_Text],
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN GETUTCDATE() ELSE NULL END,
		   CFV_Input.[File_Bytes],
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN dbo.Session_GetUserName() ELSE NULL END,
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN GETUTCDATE() ELSE NULL END
	  FROM [ConfigurationFileInstances] CFI
	       LEFT JOIN (SELECT [Instance_Name] = CFVX.[Version].[value]('@Instance_Name', 'VARCHAR(100)'),
		                     [VersionNotes_Text] = CFVX.[Version].[value]('@VersionNotes_Text', 'VARCHAR(500)'),
							 [File_Bytes] = COALESCE(CFVX.[Version].[value]('@File_Bytes', 'VARBINARY(MAX)'), CAST('' AS VARBINARY(MAX)))
		                FROM @ConfigurationFiles_Xml.[nodes]('/ConfigFiles/Version') AS CFVX(Version)) CFV_Input
				  ON CFV_Input.[Instance_Name] = CFI.[Instance_Name]
	 WHERE CFI.[ConfigurationFile_Id] = @ConfigurationFile_Id

	 DECLARE @Application_Id INT
	 SET @Application_Id = (SELECT [Application_Id]
	                          FROM [ConfigurationFiles] CF
							 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	 DECLARE @ReleaseNumbers TABLE ([Release_Number] RELEASENUMBER NOT NULL)
	 INSERT INTO @ReleaseNumbers
	 SELECT [Value_Text] FROM dbo.CsvToTable(@ReleaseNumbers_Csv, ',')

	 UPDATE RCF
	    SET RCF.[Version_Number] = @Version_Number
	   FROM [ReleaseConfigurationFiles] RCF
	        INNER JOIN [Releases] R
			        ON R.[Release_Id] = RCF.[Release_Id]
	  WHERE R.[Application_Id] = @Application_Id
	    AND R.[Release_Number] IN (SELECT [Release_Number] FROM @ReleaseNumbers)
		AND RCF.[ConfigurationFile_Id] = @ConfigurationFile_Id

	 INSERT INTO [ReleaseConfigurationFiles]
	 (
		[Release_Id],
		[ConfigurationFile_Id],
		[Version_Number]
	 )
	 SELECT R.[Release_Id],
			@ConfigurationFile_Id,
			@Version_Number
	   FROM @ReleaseNumbers RN
	        INNER JOIN [Releases] R
			        ON R.[Application_Id] = @Application_Id
				   AND R.[Release_Number] = RN.[Release_Number]
	  WHERE NOT EXISTS(SELECT *
	                     FROM [ReleaseConfigurationFiles] RCF
						WHERE RCF.[Release_Id] = R.[Release_Id]
						  AND RCF.[ConfigurationFile_Id] = @ConfigurationFile_Id)

	 EXEC [Events_RaiseEvent] 'CFGNWV', @Application_Id, @ConfigurationFile_Id, @Version_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFileVersions] TO [BuildMasterUser_Role]
GO
